@@ -295,9 +295,10 @@
 #include "photosyst.h"
 #include "showgeneric.h"
 #include "parseable.h"
+#include "json.h"
 #include "gpucom.h"
 
-#define	allflags  "ab:cde:fghijklmnopqrstuvwxyz1ABCDEFGHIJKL:MNOP:QRSTUVWXYZ"
+#define	allflags  "ab:cde:fghijklmnopqrstuvwxyz1ABCDEFGHIJ:KL:MNOP:QRSTUVWXYZ"
 #define	MAXFL		64      /* maximum number of command-line flags  */
 
 /*
@@ -325,6 +326,7 @@ char      	rmspaces   = 0;  /* boolean: remove spaces from command  */
 		                 /* name in case of parseable output     */
 
 unsigned short	hertz;
+unsigned int	pidwidth;
 unsigned int	pagesize;
 unsigned int	nrgpus;
 int 		osrel;
@@ -366,6 +368,7 @@ void do_maxifb(char *, char *);
 void do_maxnfsm(char *, char *);
 void do_maxcont(char *, char *);
 void do_maxnuma(char *, char *);
+void do_maxllc(char *, char *);
 void do_colinfo(char *, char *);
 void do_colalmost(char *, char *);
 void do_colcrit(char *, char *);
@@ -379,6 +382,7 @@ void do_ownswpline(char *, char *);
 void do_ownpagline(char *, char *);
 void do_ownmemnumaline(char *, char *);
 void do_owncpunumaline(char *, char *);
+void do_ownllcline(char *, char *);
 void do_owndskline(char *, char *);
 void do_ownnettransportline(char *, char *);
 void do_ownnetnetline(char *, char *);
@@ -417,6 +421,7 @@ static struct {
 	{	"maxlinenfsm",		do_maxnfsm,		0, },
 	{	"maxlinecont",		do_maxcont,		0, },
 	{	"maxlinenuma",		do_maxnuma,		0, },
+	{	"maxlinellc",		do_maxllc,		0, },
 	{	"colorinfo",		do_colinfo,		0, },
 	{	"coloralmost",		do_colalmost,		0, },
 	{	"colorcritical",	do_colcrit,		0, },
@@ -429,6 +434,7 @@ static struct {
 	{	"ownpagline",		do_ownpagline,		0, },
 	{	"ownmemnumaline",	do_ownmemnumaline,	0, },
 	{	"ownnumacpuline",	do_owncpunumaline,	0, },
+	{	"ownllcline",		do_ownllcline,		0, },
 	{	"owndskline",		do_owndskline,		0, },
 	{	"ownnettrline",		do_ownnettransportline,	0, },
 	{	"ownnetnetline",	do_ownnetnetline,	0, },
@@ -583,6 +589,13 @@ main(int argc, char *argv[])
 				vis.show_samp = parseout;
 				break;
 
+                           case 'J':		/* json output?          */
+				if ( !jsondef(optarg) )
+					prusage(argv[0]);
+
+				vis.show_samp = jsonout;
+				break;
+
                            case 'L':		/* line length                */
 				if ( !numeric(optarg) )
 					prusage(argv[0]);
@@ -652,6 +665,7 @@ main(int argc, char *argv[])
 	*/
 	hertz		= sysconf(_SC_CLK_TCK);
 	pagesize	= sysconf(_SC_PAGESIZE);
+	pidwidth	= getpidwidth();
 
 	/*
 	** check if raw data from a file must be viewed
@@ -1083,6 +1097,7 @@ prusage(char *myname)
 	                MCALCPSS);
 	printf("\t  -%c  determine WCHAN (string) per thread\n", MGETWCHAN);
 	printf("\t  -P  generate parseable output for specified label(s)\n");
+	printf("\t  -J  generate JSON output for specified label(s)\n");
 	printf("\t  -%c  no spaces in parseable output for command (line)\n",
 			MRMSPACES);
 	printf("\t  -L  alternate line length (default 80) in case of "
