@@ -59,6 +59,7 @@ struct netpertask;
 #define RRIOSTAT	0x0020
 #define RRDOCKSTAT	0x0040
 #define RRGPUSTAT	0x0080
+#define RRCGRSTAT	0x0100
 
 struct visualize {
 	char	(*show_samp)  (time_t, int,
@@ -99,6 +100,7 @@ extern int      	ossub;
 
 extern unsigned short	hertz;
 extern unsigned int	pagesize;
+extern unsigned int	pidwidth;
 extern unsigned int	nrgpus;
 
 extern int		supportflags;
@@ -120,6 +122,7 @@ extern int		almostcrit;
 #define	NETATOPD	0x00000020
 #define	DOCKSTAT	0x00000040
 #define	GPUSTAT		0x00000080
+#define	CGROUPV2	0x00000100
 
 /*
 ** in rawlog file, the four least significant bits 
@@ -172,6 +175,7 @@ int		nfsmcompar(const void *, const void *);
 int		contcompar(const void *, const void *);
 int		memnumacompar(const void *, const void *);
 int		cpunumacompar(const void *, const void *);
+int		llccompar(const void *, const void *);
 
 int  		rawread(void);
 char		rawwrite (time_t, int,
@@ -186,6 +190,7 @@ unsigned short 	getnumvers(void);
 void		ptrverify(const void *, const char *, ...);
 void		mcleanstop(int, const char *, ...);
 void		cleanstop(int);
+int		getpidwidth(void);
 void		prusage(char *);
 
 int		droprootprivs(void);
@@ -201,3 +206,4 @@ void		netatop_exiterase(void);
 void		netatop_exithash(char);
 void		netatop_exitfind(unsigned long, struct tstat *, struct tstat *);
 void		set_oom_score_adj(void);
+int		run_in_guest(void);
