@@ -133,7 +133,8 @@ atopsar(int argc, char *argv[])
 		*/
 		flaglist = malloc(pricnt+32);
 
-		ptrverify(flaglist, "Malloc failed for %d flags\n", pricnt+32);
+		if (!flaglist)
+			ptrverify(flaglist, "Malloc failed for %d flags\n", pricnt+32);
 
 		for (i=0; i < pricnt; i++)
 			flaglist[i] = pridef[i].flag;
@@ -203,7 +204,7 @@ atopsar(int argc, char *argv[])
 			   case 'H':		/* repeat headers          */
 				repeathead = 23;	/* define default  */
 
-				if (isatty(1))
+				if (isatty(fileno(stdout)))
 				{
 					struct winsize wsz;
 
@@ -298,7 +299,7 @@ atopsar(int argc, char *argv[])
 	*/
 	if (usecolors == 't')
 	{
-		if (! isatty(1) )
+		if (! isatty(fileno(stdout)) )
 			usecolors = 0;
 	}
 
@@ -1168,7 +1169,7 @@ do_atopsarflags(char *name, char *val)
 		   case 'H':		/* repeat headers          */
 			repeathead = 23;	/* define default  */
 
-			if (isatty(1))
+			if (isatty(fileno(stdout)))
 			{
 				struct winsize wsz;
 
