@@ -312,6 +312,8 @@ deviattask(struct tstat    *curtpres, unsigned long ntaskpres,
 
 		strcpy(devstat->gen.cmdline, prestat.gen.cmdline);
 
+		devstat->cpu.curcpu = -1;
+
 		/*
 		** due to the strange exponent-type storage of values
 		** in the process accounting record, the resource-value
@@ -444,27 +446,35 @@ calcdiff(struct tstat *devstat, const struct tstat *curstat,
 	if (newtask)
 		devstat->gen.excode |= ~(INT_MAX);
 
-	devstat->cpu.nice     = curstat->cpu.nice;
-	devstat->cpu.prio     = curstat->cpu.prio;
-	devstat->cpu.rtprio   = curstat->cpu.rtprio;
-	devstat->cpu.policy   = curstat->cpu.policy;
-	devstat->cpu.curcpu   = curstat->cpu.curcpu;
-	devstat->cpu.sleepavg = curstat->cpu.sleepavg;
+	devstat->cpu.nice        = curstat->cpu.nice;
+	devstat->cpu.prio        = curstat->cpu.prio;
+	devstat->cpu.rtprio      = curstat->cpu.rtprio;
+	devstat->cpu.policy      = curstat->cpu.policy;
+	devstat->cpu.curcpu      = curstat->cpu.curcpu;
+	devstat->cpu.sleepavg    = curstat->cpu.sleepavg;
+	devstat->cpu.cgcpuweight = curstat->cpu.cgcpuweight;
+	devstat->cpu.cgcpumax    = curstat->cpu.cgcpumax;
+	devstat->cpu.cgcpumaxr   = curstat->cpu.cgcpumaxr;
 
 	if (curstat->cpu.wchan[0])
 		strcpy(devstat->cpu.wchan, curstat->cpu.wchan);
 	else
 		devstat->cpu.wchan[0] = 0;
 
-	devstat->mem.vexec    = curstat->mem.vexec;
-	devstat->mem.vmem     = curstat->mem.vmem;
-	devstat->mem.rmem     = curstat->mem.rmem;
-	devstat->mem.pmem     = curstat->mem.pmem;
-	devstat->mem.vdata    = curstat->mem.vdata;
-	devstat->mem.vstack   = curstat->mem.vstack;
-	devstat->mem.vlibs    = curstat->mem.vlibs;
-	devstat->mem.vswap    = curstat->mem.vswap;
-	devstat->mem.vlock    = curstat->mem.vlock;
+	devstat->mem.vexec  = curstat->mem.vexec;
+	devstat->mem.vmem   = curstat->mem.vmem;
+	devstat->mem.rmem   = curstat->mem.rmem;
+	devstat->mem.pmem   = curstat->mem.pmem;
+	devstat->mem.vdata  = curstat->mem.vdata;
+	devstat->mem.vstack = curstat->mem.vstack;
+	devstat->mem.vlibs  = curstat->mem.vlibs;
+	devstat->mem.vswap  = curstat->mem.vswap;
+	devstat->mem.vlock  = curstat->mem.vlock;
+
+	devstat->mem.cgmemmax  = curstat->mem.cgmemmax;
+	devstat->mem.cgmemmaxr = curstat->mem.cgmemmaxr;
+	devstat->mem.cgswpmax  = curstat->mem.cgswpmax;
+	devstat->mem.cgswpmaxr = curstat->mem.cgswpmaxr;
 
 	if (curstat->gpu.state || prestat->gpu.state) // GPU use?
 	{
@@ -516,6 +526,9 @@ calcdiff(struct tstat *devstat, const struct tstat *curstat,
 	devstat->cpu.rundelay  =
 		subcount(curstat->cpu.rundelay, prestat->cpu.rundelay);
 
+	devstat->cpu.blkdelay  =
+		subcount(curstat->cpu.blkdelay, prestat->cpu.blkdelay);
+
 	devstat->dsk.rio    =
 		subcount(curstat->dsk.rio, prestat->dsk.rio);
 	devstat->dsk.rsz    =
@@ -686,6 +699,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 	dev->mem.totswap	= cur->mem.totswap;
 	dev->mem.freeswap	= cur->mem.freeswap;
 	dev->mem.swapcached	= cur->mem.swapcached;
+	dev->mem.pagetables	= cur->mem.pagetables;
 
 	dev->mem.shmem		= cur->mem.shmem;
 	dev->mem.shmrss		= cur->mem.shmrss;
@@ -702,6 +716,11 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 	dev->mem.zswstored	= cur->mem.zswstored;
 	dev->mem.zswtotpool	= cur->mem.zswtotpool;
 
+	dev->mem.tcpsock	= cur->mem.tcpsock;
+	dev->mem.udpsock	= cur->mem.udpsock;
+
+	dev->mem.pgouts		= subcount(cur->mem.pgouts,  pre->mem.pgouts);
+	dev->mem.pgins		= subcount(cur->mem.pgins,   pre->mem.pgins);
 	dev->mem.swouts		= subcount(cur->mem.swouts,  pre->mem.swouts);
 	dev->mem.swins		= subcount(cur->mem.swins,   pre->mem.swins);
 	dev->mem.pgscans	= subcount(cur->mem.pgscans, pre->mem.pgscans);
@@ -723,6 +742,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 	for (i=0; i < dev->memnuma.nrnuma; i++)
 	{
+		dev->memnuma.numa[i].numanr      = cur->memnuma.numa[i].numanr;
 		dev->memnuma.numa[i].totmem      = cur->memnuma.numa[i].totmem;
 		dev->memnuma.numa[i].freemem     = cur->memnuma.numa[i].freemem;
 		dev->memnuma.numa[i].filepage    = cur->memnuma.numa[i].filepage;
@@ -743,6 +763,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 		for (i=0; i < dev->cpunuma.nrnuma; i++)
 		{
 			dev->cpunuma.numa[i].nrcpu  = cur->cpunuma.numa[i].nrcpu;
+			dev->cpunuma.numa[i].numanr = cur->cpunuma.numa[i].numanr;
 
 			dev->cpunuma.numa[i].utime  = subcount(cur->cpunuma.numa[i].utime,
 								pre->cpunuma.numa[i].utime);
@@ -1010,6 +1031,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 		                                  pre->dsk.dsk[j].nwrite);
 		dev->dsk.dsk[i].nwsect = subcount(cur->dsk.dsk[i].nwsect,
 		                                  pre->dsk.dsk[j].nwsect);
+		dev->dsk.dsk[i].inflight  = cur->dsk.dsk[i].inflight;
 		dev->dsk.dsk[i].io_ms  = subcount(cur->dsk.dsk[i].io_ms,
 		                                  pre->dsk.dsk[j].io_ms);
 		dev->dsk.dsk[i].avque  = subcount(cur->dsk.dsk[i].avque,
@@ -1327,17 +1349,16 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 		dev->cfs.cont[i].ctid    = cur->cfs.cont[i].ctid;
 		dev->cfs.cont[i].numproc = cur->cfs.cont[i].numproc;
+		dev->cfs.cont[i].physpages = cur->cfs.cont[i].physpages;
 
 		dev->cfs.cont[i].system  = subcount(cur->cfs.cont[i].system,
-		                                    pre->cfs.cont[i].system);
+		                                    pre->cfs.cont[j].system);
 		dev->cfs.cont[i].user    = subcount(cur->cfs.cont[i].user,
-		                                    pre->cfs.cont[i].user);
+		                                    pre->cfs.cont[j].user);
 		dev->cfs.cont[i].nice    = subcount(cur->cfs.cont[i].nice,
-		                                    pre->cfs.cont[i].nice);
+		                                    pre->cfs.cont[j].nice);
 		dev->cfs.cont[i].uptime  = subcount(cur->cfs.cont[i].uptime,
-		                                    pre->cfs.cont[i].uptime);
-
-		dev->cfs.cont[i].physpages = cur->cfs.cont[i].physpages;
+		                                    pre->cfs.cont[j].uptime);
 	}
 
 	dev->cfs.nrcontainer = cur->cfs.nrcontainer;
@@ -1407,8 +1428,20 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 
 	dev->ifb.nrports = cur->ifb.nrports;
 
+	/*
+	** calculate deviations for Last Level Cache
+	*/
+	for (i = 0; i < cur->llc.nrllcs; i++)
+	{
+		dev->llc.perllc[i].id        = cur->llc.perllc[i].id;
+		dev->llc.perllc[i].occupancy = cur->llc.perllc[i].occupancy;
+		dev->llc.perllc[i].mbm_local = cur->llc.perllc[i].mbm_local -
+					       pre->llc.perllc[i].mbm_local;
+		dev->llc.perllc[i].mbm_total = cur->llc.perllc[i].mbm_total -
+					       pre->llc.perllc[i].mbm_total;
+	}
 	
-
+	dev->llc.nrllcs = cur->llc.nrllcs;
 
 #if	HTTPSTATS
 	/*
@@ -1499,11 +1532,17 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 		tot->mem.totswap	 = new->mem.totswap;
 		tot->mem.freeswap	 = new->mem.freeswap;
 		tot->mem.swapcached	 = new->mem.swapcached;
+		tot->mem.pagetables	 = new->mem.pagetables;
 
 		tot->mem.shmem		 = new->mem.shmem;
 		tot->mem.shmrss		 = new->mem.shmrss;
 		tot->mem.shmswp		 = new->mem.shmswp;
 
+		tot->mem.tcpsock	= new->mem.tcpsock;
+		tot->mem.udpsock	= new->mem.udpsock;
+
+		tot->mem.pgouts		+= new->mem.pgouts;
+		tot->mem.pgins		+= new->mem.pgins;
 		tot->mem.swouts		+= new->mem.swouts;
 		tot->mem.swins		+= new->mem.swins;
 		tot->mem.pgscans	+= new->mem.pgscans;
