@@ -57,6 +57,7 @@ void 	print_NET();
 void 	print_IFB();
 void 	print_NUM();
 void 	print_NUC();
+void 	print_LLC();
 
 void 	print_PRG();
 void 	print_PRC();
@@ -66,6 +67,7 @@ void 	print_PRN();
 void 	print_PRE();
 
 static char *spaceformat(char *, char *);
+static int  cgroupv2max(int, int);
 
 /*
 ** table with possible labels and the corresponding
@@ -96,6 +98,7 @@ static struct labeldef	labeldef[] = {
 	{ "IFB",	0,	print_IFB },
 	{ "NUM",	0,	print_NUM },
 	{ "NUC",	0,	print_NUC },
+	{ "LLC",	0,	print_LLC },
 
 	{ "PRG",	0,	print_PRG },
 	{ "PRC",	0,	print_PRC },
@@ -384,7 +387,8 @@ print_GPU(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 void
 print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	printf(	"%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf(	"%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
+   		"%lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.physmem,
@@ -403,7 +407,10 @@ print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         		ss->mem.freehugepage,
         		ss->mem.zfsarcsize != -1 ? ss->mem.zfsarcsize : 0,
         		ss->mem.ksmsharing != -1 ? ss->mem.ksmsharing : 0,
-        		ss->mem.ksmshared  != -1 ? ss->mem.ksmshared  : 0);
+        		ss->mem.ksmshared  != -1 ? ss->mem.ksmshared  : 0,
+			ss->mem.tcpsock,
+			ss->mem.udpsock,
+   			ss->mem.pagetables);
 }
 
 void
@@ -425,7 +432,7 @@ print_SWP(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 void
 print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.pgscans,
@@ -436,7 +443,9 @@ print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->mem.oomkills,
 			ss->mem.compactstall,
 			ss->mem.pgmigrate,
-			ss->mem.numamigrate);
+			ss->mem.numamigrate,
+			ss->mem.pgins,
+			ss->mem.pgouts);
 }
 
 void
@@ -464,7 +473,7 @@ print_LVM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
         for (i=0; ss->dsk.lvm[i].name[0]; i++)
 	{
-		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld\n",
+		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %.2f\n",
 			hp,
 			ss->dsk.lvm[i].name,
 			ss->dsk.lvm[i].io_ms,
@@ -473,7 +482,11 @@ print_LVM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->dsk.lvm[i].nwrite,
 			ss->dsk.lvm[i].nwsect,
 			ss->dsk.lvm[i].ndisc,
-			ss->dsk.lvm[i].ndsect);
+			ss->dsk.lvm[i].ndsect,
+			ss->dsk.lvm[i].inflight,
+			ss->dsk.lvm[i].io_ms > 0 ?
+			  (double)ss->dsk.lvm[i].avque/ss->dsk.lvm[i].io_ms :
+			  0.0);
 	}
 }
 
@@ -484,7 +497,7 @@ print_MDD(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
         for (i=0; ss->dsk.mdd[i].name[0]; i++)
 	{
-		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld\n",
+		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %.2f\n",
 			hp,
 			ss->dsk.mdd[i].name,
 			ss->dsk.mdd[i].io_ms,
@@ -493,7 +506,11 @@ print_MDD(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->dsk.mdd[i].nwrite,
 			ss->dsk.mdd[i].nwsect,
 			ss->dsk.mdd[i].ndisc,
-			ss->dsk.mdd[i].ndsect);
+			ss->dsk.mdd[i].ndsect,
+			ss->dsk.mdd[i].inflight,
+			ss->dsk.mdd[i].io_ms > 0 ?
+			  (double)ss->dsk.mdd[i].avque/ss->dsk.mdd[i].io_ms :
+			  0.0);
 	}
 }
 
@@ -504,7 +521,7 @@ print_DSK(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
         for (i=0; ss->dsk.dsk[i].name[0]; i++)
 	{
-		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld\n",
+		printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %.2f\n",
 			hp,
 			ss->dsk.dsk[i].name,
 			ss->dsk.dsk[i].io_ms,
@@ -513,7 +530,11 @@ print_DSK(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->dsk.dsk[i].nwrite,
 			ss->dsk.dsk[i].nwsect,
 			ss->dsk.dsk[i].ndisc,
-			ss->dsk.dsk[i].ndsect);
+			ss->dsk.dsk[i].ndsect,
+			ss->dsk.dsk[i].inflight,
+			ss->dsk.dsk[i].io_ms > 0 ?
+			  (double)ss->dsk.dsk[i].avque/ss->dsk.dsk[i].io_ms :
+			  0.0);
 	}
 }
 
@@ -648,7 +669,7 @@ print_NUM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < ss->memnuma.nrnuma; i++)
 	{
 		printf(	"%s %d %u %.0f %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
-			hp, i,
+			hp, ss->memnuma.numa[i].numanr,
 			pagesize,
 			ss->memnuma.numa[i].frag * 100.0,
 			ss->memnuma.numa[i].totmem,
@@ -672,7 +693,7 @@ print_NUC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < ss->cpunuma.nrnuma; i++)
 	{
 		printf(	"%s %d %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
-			hp, i,
+			hp, ss->cpunuma.numa[i].numanr,
 	        	ss->cpunuma.numa[i].nrcpu,
 	        	ss->cpunuma.numa[i].stime,
         		ss->cpunuma.numa[i].utime,
@@ -686,6 +707,21 @@ print_NUC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	}
 }
 
+void
+print_LLC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
+{
+	register int 	i;
+
+	for (i=0; i < ss->llc.nrllcs; i++)
+	{
+		printf(	"%s LLC%03d %3.1f%% %lld %lld\n",
+			hp,
+			ss->llc.perllc[i].id,
+			ss->llc.perllc[i].occupancy * 100,
+			ss->llc.perllc[i].mbm_total,
+			ss->llc.perllc[i].mbm_local);
+	}
+}
 
 /*
 ** print functions for process-level statistics
@@ -694,7 +730,8 @@ void
 print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
 	register int	i, exitcode;
-	char		namout[PNAMLEN+1+2], cmdout[CMDLEN+1+2];
+	char		namout[PNAMLEN+1+2], cmdout[CMDLEN+1+2],
+			pathout[CGRLEN+1];
 
 	for (i=0; i < nact; i++, ps++)
 	{
@@ -704,7 +741,7 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			exitcode = (ps->gen.excode >>   8) & 0xff;
 
 		printf("%s %d %s %c %d %d %d %d %d %ld %s %d %d %d %d "
- 		       "%d %d %d %d %d %d %ld %c %d %d %s %c\n",
+ 		       "%d %d %d %d %d %d %ld %c %d %d %s %c %s\n",
 			hp,
 			ps->gen.pid,
 			spaceformat(ps->gen.name, namout),
@@ -731,7 +768,8 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.vpid,
 			ps->gen.ctid,
 			ps->gen.container[0] ? ps->gen.container:"-",
-        		ps->gen.excode & ~(INT_MAX) ? 'N' : '-');
+        		ps->gen.excode & ~(INT_MAX) ? 'N' : '-',
+			spaceformat(ps->gen.cgpath, pathout));
 	}
 }
 
@@ -744,7 +782,7 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < nact; i++, ps++)
 	{
 		printf("%s %d %s %c %u %lld %lld %d %d %d %d %d %d %d %c "
-		       "%llu %s\n",
+		       "%llu %s %llu %d %d\n",
 			hp,
 			ps->gen.pid,
 			spaceformat(ps->gen.name, namout),
@@ -761,7 +799,10 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.tgid,
 			ps->gen.isproc ? 'y':'n',
 			ps->cpu.rundelay,
-			spaceformat(ps->cpu.wchan, wchanout));
+			spaceformat(ps->cpu.wchan, wchanout),
+			ps->cpu.blkdelay,
+			cgroupv2max(ps->gen.isproc, ps->cpu.cgcpumax),
+			cgroupv2max(ps->gen.isproc, ps->cpu.cgcpumaxr));
 	}
 }
 
@@ -774,7 +815,7 @@ print_PRM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < nact; i++, ps++)
 	{
 		printf("%s %d %s %c %u %lld %lld %lld %lld %lld %lld "
-		       "%lld %lld %lld %lld %lld %d %c %lld %lld\n",
+		       "%lld %lld %lld %lld %lld %d %c %lld %lld %d %d %d %d\n",
 			hp,
 			ps->gen.pid,
 			spaceformat(ps->gen.name, namout),
@@ -795,7 +836,11 @@ print_PRM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.isproc ? 'y':'n',
 			ps->mem.pmem == (unsigned long long)-1LL ?
 							0:ps->mem.pmem,
-			ps->mem.vlock);
+			ps->mem.vlock,
+			cgroupv2max(ps->gen.isproc, ps->mem.cgmemmax),
+			cgroupv2max(ps->gen.isproc, ps->mem.cgmemmaxr),
+			cgroupv2max(ps->gen.isproc, ps->mem.cgswpmax),
+			cgroupv2max(ps->gen.isproc, ps->mem.cgswpmaxr));
 	}
 }
 
@@ -920,3 +965,18 @@ spaceformat(char *istr, char *ostr)
 
 	return ostr;
 }
+
+/*
+** return proper integer for cgroup v2 maximum values
+*/
+static int
+cgroupv2max(int isproc, int max)
+{
+	if (! (supportflags&CGROUPV2))
+		return -3;
+
+	if (! isproc)
+		return -2;
+
+	return max;
+}
