@@ -78,29 +78,13 @@
 /* recognize numa node */
 #define	NUMADIR	"/sys/devices/system/node"
 
+/* recognize LLC monitor data */
+#define LLCDIR	"/sys/fs/resctrl/mon_data"
+#define L3SIZE	"/sys/devices/system/cpu/cpu0/cache/index3/size"
+
 /* Refer to mmzone.h, the default is 11 */
 #define	MAX_ORDER	11
 
-/* hypervisor enum */
-enum {
-	HYPER_NONE	= 0,
-	HYPER_XEN,
-	HYPER_KVM,
-	HYPER_MSHV,
-	HYPER_VMWARE,
-	HYPER_IBM,
-	HYPER_VSERVER,
-	HYPER_UML,
-	HYPER_INNOTEK,
-	HYPER_HITACHI,
-	HYPER_PARALLELS,
-	HYPER_VBOX,
-	HYPER_OS400,
-	HYPER_PHYP,
-	HYPER_SPAR,
-	HYPER_WSL,
-};
-
 #ifndef	NOPERFEVENT
 enum {
 	PERF_EVENTS_AUTO = 0,
@@ -110,9 +94,6 @@ enum {
 
 static int	perfevents = PERF_EVENTS_AUTO;
 static void	getperfevents(struct cpustat *);
-
-static int	run_in_guest(void);
-static int	get_hypervisor(void);
 #endif
 
 static int	get_infiniband(struct ifbstat *);
@@ -228,7 +209,8 @@ numa_allocate_cpumask()
 	struct bitmask *bmp;
 
 	bmp = malloc(sizeof(*bmp));
-	ptrverify(bmp, "Malloc failed for numa bitmask");
+	if (!bmp)
+		ptrverify(bmp, "Malloc failed for numa bitmask");
 
 	bmp->size = ncpus;
 	bmp->maskp = calloc(longsperbits(ncpus), sizeof(unsigned long));
@@ -451,7 +433,7 @@ photosyst(struct sstat *si)
         **
 	** store them in binary form
 	*/
-        static char fn[256];
+        static char fn[512];
         int didone=0;
 
 	if (si->cpu.nrcpu <= SCALINGMAXCPU)
@@ -585,6 +567,19 @@ photosyst(struct sstat *si)
 			if (nr < 2)		/* headerline ? --> skip */
 				continue;
 
+			/* pgpgin & pgpgout fields in KB from vmstat */
+			if ( strcmp("pgpgin", nam) == EQ)
+			{
+				si->mem.pgins   = cnts[0] * 1024 / pagesize;
+				continue;
+			}
+
+			if ( strcmp("pgpgout", nam) == EQ)
+			{
+				si->mem.pgouts   = cnts[0] * 1024 / pagesize;
+				continue;
+			}
+
 			if ( strcmp("pswpin", nam) == EQ)
 			{
 				si->mem.swins   = cnts[0];
@@ -663,6 +658,7 @@ photosyst(struct sstat *si)
 	si->mem.freeswap 	= (count_t)-1;
 	si->mem.swapcached 	= (count_t) 0;
 	si->mem.committed 	= (count_t) 0;
+	si->mem.pagetables 	= (count_t) 0;
 
 	if ( (fp = fopen("meminfo", "r")) != NULL)
 	{
@@ -783,18 +779,25 @@ photosyst(struct sstat *si)
 				{
 					si->mem.hugepagesz = cnts[0]*1024;
 				}
+			else	if (strcmp("PageTables:", nam) == EQ)
+				{
+					si->mem.pagetables = cnts[0]*1024/
+								pagesize;
+				}
 		}
 
 		fclose(fp);
 	}
 
 	/*
-	** gather vmware-related statistics from /proc/vmmemctl
-	** (only present if balloon driver enabled)
+	** gather vmware-related statistics from /sys/kernel/debug/vmmemctl
+	** (only present if balloon driver enabled) or from /proc/vmmemctl
+	** for older balloon driver implementations
 	*/ 
 	si->mem.vmwballoon = (count_t) -1;
 
-	if ( (fp = fopen("vmmemctl", "r")) != NULL)
+	if ( (fp = fopen("/sys/kernel/debug/vmmemctl", "r")) != NULL ||
+	     (fp = fopen("/proc/vmmemctl",             "r")) != NULL   )
 	{
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
@@ -873,6 +876,7 @@ photosyst(struct sstat *si)
 					if (cnts[0] != j)
 						continue;
 
+					si->memnuma.numa[j].numanr = j;
 					if ( strcmp("MemTotal:", nam) == EQ)
 						si->memnuma.numa[j].totmem = cnts[1]*1024/pagesize;
 					else if ( strcmp("MemFree:", nam) == EQ)
@@ -1016,6 +1020,7 @@ photosyst(struct sstat *si)
 					si->cpunuma.numa[j].guest += si->cpu.cpu[i].guest;
 				}
 			}
+			si->cpunuma.numa[j].numanr = j;
 		}
 
 		free(line);
@@ -1031,6 +1036,7 @@ photosyst(struct sstat *si)
  	** 	- interface stats from the file /proc/net/dev
  	** 	- IPv4      stats from the file /proc/net/snmp
  	** 	- IPv6      stats from the file /proc/net/snmp6
+ 	** 	- sock mem  stats from the file /proc/net/sockstat
 	*/
 
 	/*
@@ -1211,6 +1217,38 @@ photosyst(struct sstat *si)
 		fclose(fp);
 	}
 
+	/*
+	** IP version 4: TCP & UDP memory allocations.
+	*/
+	if ( (fp = fopen("net/sockstat", "r")) != NULL)
+	{
+		char tcpmem[16], udpmem[16];
+
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			nr = sscanf(linebuf,
+				"%15s %*s %*d %s %lld %*s %*d %*s %*d %s %lld\n",
+				nam, udpmem, &cnts[0], tcpmem, &cnts[1]);
+
+			if ( strcmp("TCP:", nam) == 0)
+			{
+				if ( strcmp("mem", tcpmem) == 0) {
+					si->mem.tcpsock = cnts[1];
+				}
+				continue;
+			}
+
+			if ( strcmp("UDP:", nam) == 0)
+			{
+				if ( strcmp("mem", udpmem) == 0) {
+					si->mem.udpsock = cnts[0];
+				}
+				continue;
+			}
+		}
+		fclose(fp);
+	}
+
 	/*
 	** check if extended partition-statistics are provided < kernel 2.6
 	*/
@@ -1224,12 +1262,13 @@ photosyst(struct sstat *si)
 		{
 			nr = sscanf(linebuf,
 			      "%*d %*d %*d %255s %lld %*d %lld %*d "
-			      "%lld %*d %lld %*d %*d %lld %lld",
+			      "%lld %*d %lld %*d %lld %lld %lld",
 			        diskname,
 				&(si->dsk.dsk[i].nread),
 				&(si->dsk.dsk[i].nrsect),
 				&(si->dsk.dsk[i].nwrite),
 				&(si->dsk.dsk[i].nwsect),
+				&(si->dsk.dsk[i].inflight),
 				&(si->dsk.dsk[i].io_ms),
 				&(si->dsk.dsk[i].avque) );
 
@@ -1238,7 +1277,7 @@ photosyst(struct sstat *si)
 			** or just one of the partitions of a disk (to be
 			** skipped)
 			*/
-			if (nr == 7)	/* full stats-line ? */
+			if (nr == 8)	/* full stats-line ? */
 			{
 				if ( isdisk(0, 0, diskname,
 				                 &(si->dsk.dsk[i]),
@@ -1281,15 +1320,15 @@ photosyst(struct sstat *si)
 			      "%d %d %255s "		// ident
                               "%lld %*d %lld %*d "	// reads
 			      "%lld %*d %lld %*d "	// writes
-			      "%*d %lld %lld "		// misc
+			      "%lld %lld %lld "		// misc
 			      "%lld %*d %lld %*d",	// discards
 				&major, &minor, diskname,
 				&tmpdsk.nread,  &tmpdsk.nrsect,
 				&tmpdsk.nwrite, &tmpdsk.nwsect,
-				&tmpdsk.io_ms,  &tmpdsk.avque,
+				&tmpdsk.inflight, &tmpdsk.io_ms, &tmpdsk.avque,
 				&tmpdsk.ndisc,  &tmpdsk.ndsect);
 
-			if (nr >= 9)	/* full stats-line ? */
+			if (nr >= 10)	/* full stats-line ? */
 			{
 				/*
   				** when no transfers issued, skip disk (partition)
@@ -1767,6 +1806,84 @@ photosyst(struct sstat *si)
 		}
 	}
 
+	/*
+	** gather per LLC related statistics from the file
+	** /sys/fs/resctrl/mon_data/mon_L3_XX
+	*/
+	dirp = opendir(LLCDIR);
+
+	if (dirp)
+	{
+		static int l3_cache_size;
+
+		if (!l3_cache_size)
+		{
+			if ( (fp = fopen(L3SIZE, "r")) != NULL)
+			{
+				if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+				{
+					sscanf(linebuf, "%uK\n", &l3_cache_size);
+					l3_cache_size *= 1024;
+				}
+
+				fclose(fp);
+			}
+		}
+
+		/*
+		** walk the LLC directory, gather each LLC
+		*/
+		while ( (dentry = readdir(dirp)) )
+		{
+			struct perllc *llc = &si->llc.perllc[si->llc.nrllcs];
+			unsigned long llc_occupancy;
+
+			if (strncmp(dentry->d_name, "mon_L3_", 7))
+				continue;
+
+			/* get cache id from directory name like mon_L3_00 */
+			sscanf(dentry->d_name + 7, "%hhd\n", &llc->id);
+
+			snprintf(fn, sizeof fn, LLCDIR "/%s/llc_occupancy", dentry->d_name);
+			if ( (fp = fopen(fn, "r")) != NULL)
+			{
+				if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+				{
+					sscanf(linebuf, "%lu\n", &llc_occupancy);
+					llc->occupancy = (float)llc_occupancy / l3_cache_size;
+				}
+
+				fclose(fp);
+			}
+
+			snprintf(fn, sizeof fn, LLCDIR "/%s/mbm_local_bytes", dentry->d_name);
+			if ( (fp = fopen(fn, "r")) != NULL)
+			{
+				if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+				{
+					sscanf(linebuf, "%llu\n", &llc->mbm_local);
+				}
+
+				fclose(fp);
+			}
+
+			snprintf(fn, sizeof fn, LLCDIR "/%s/mbm_total_bytes", dentry->d_name);
+			if ( (fp = fopen(fn, "r")) != NULL)
+			{
+				if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+				{
+					sscanf(linebuf, "%llu\n", &llc->mbm_total);
+				}
+				fclose(fp);
+			}
+
+			if (++si->llc.nrllcs >= MAXLLC) /* too many LLC ? */
+				break;
+		}
+
+		closedir(dirp);
+	}
+
 	/*
  	** verify presence of InfiniBand controllers
 	** warning: possibly switches to other directory
@@ -1972,6 +2089,8 @@ static struct {
 	{ "^dasd[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "^mmcblk[0-9][0-9]*$",		{0},  nullmodname, DSKTYPE, },
 	{ "^emcpower[a-z][a-z]*$",		{0},  nullmodname, DSKTYPE, },
+	{ "^rbd[0-9][0-9]*$",			{0},  nullmodname, DSKTYPE, },
+	{ "^rbd[0-9][0-9]*p[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
 };
 
 static int
@@ -2723,73 +2842,6 @@ getperfevents(struct cpustat *cs)
         }
 }
 
-static int
-run_in_guest(void)
-{
-	return get_hypervisor() != HYPER_NONE;
-}
-
-#if defined(__x86_64__) || defined(__i386__)
-#define HYPERVISOR_INFO_LEAF   0x40000000
-
-static inline void
-x86_cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx,
-			 unsigned int *ecx, unsigned int *edx)
-{
-	__asm__(
-#if defined(__PIC__) && defined(__i386__)
-		"xchg %%ebx, %%esi;"
-		"cpuid;"
-		"xchg %%esi, %%ebx;"
-		: "=S" (*ebx),
-#else
-		"cpuid;"
-		: "=b" (*ebx),
-#endif
-		  "=a" (*eax),
-		  "=c" (*ecx),
-		  "=d" (*edx)
-		: "1" (op), "c"(0));
-}
-
-static int
-get_hypervisor(void)
-{
-	unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0, hyper = HYPER_NONE;
-	char hyper_vendor_id[13];
-
-	memset(hyper_vendor_id, 0, sizeof(hyper_vendor_id));
-
-	x86_cpuid(HYPERVISOR_INFO_LEAF, &eax, &ebx, &ecx, &edx);
-	memcpy(hyper_vendor_id + 0, &ebx, 4);
-	memcpy(hyper_vendor_id + 4, &ecx, 4);
-	memcpy(hyper_vendor_id + 8, &edx, 4);
-	hyper_vendor_id[12] = '\0';
-
-	if (!hyper_vendor_id[0])
-		return hyper;
-
-	if (!strncmp("XenVMMXenVMM", hyper_vendor_id, 12))
-		hyper = HYPER_XEN;
-	else if (!strncmp("KVMKVMKVM", hyper_vendor_id, 9))
-		hyper = HYPER_KVM;
-	else if (!strncmp("Microsoft Hv", hyper_vendor_id, 12))
-		hyper = HYPER_MSHV;
-	else if (!strncmp("VMwareVMware", hyper_vendor_id, 12))
-		hyper = HYPER_VMWARE;
-	else if (!strncmp("UnisysSpar64", hyper_vendor_id, 12))
-		hyper = HYPER_SPAR;
-
-	return hyper;
-}
-#else /* ! (__x86_64__ || __i386__) */
-static int
-get_hypervisor(void)
-{
-	return HYPER_NONE;
-}
-#endif
-
 #else /* ! NOPERFEVENT */
 void
 do_perfevents(char *tagname, char *tagvalue)
