@@ -38,6 +38,7 @@
 #define	MAXGPU		32
 #define	MAXGPUBUS	12
 #define	MAXGPUTYPE	12
+#define	MAXLLC		256
 
 #define	MAXDKNAM	32
 #define	MAXIBNAME	12
@@ -59,6 +60,8 @@ struct	memstat {
 	count_t	allocstall;	// try to free pages forced
 	count_t	swouts;		// number of pages swapped out
 	count_t	swins;		// number of pages swapped in
+	count_t	tcpsock;	// number of pages allocated by TCP sockets
+	count_t	udpsock;	// number of pages allocated by UDP sockets
 
 	count_t	commitlim;	// commit limit in pages
 	count_t	committed;	// number of reserved pages
@@ -84,12 +87,16 @@ struct	memstat {
 	count_t	compactstall;	// counter for process stalls
 	count_t	pgmigrate;	// counter for migrated successfully (pages)
 	count_t	numamigrate;	// counter for numa migrated (pages)
-	count_t	cfuture[9];	// reserved for future use
+	count_t	pgouts;		// total number of pages written to block device
+	count_t	pgins;		// total number of pages read from block device
+	count_t	pagetables;	// page tables of processes (pages)
+	count_t	cfuture[4];	// reserved for future use
 };
 
 /************************************************************************/
 
 struct	mempernuma {
+	int	numanr;
 	float	frag;		// fragmentation level for this numa
 	count_t	totmem;		// number of physical pages for this numa
 	count_t	freemem;	// number of free     pages for this numa
@@ -111,6 +118,7 @@ struct	memnuma {
 };
 
 struct	cpupernuma {
+	int	numanr;
 	count_t	nrcpu;		// number of cpu's
 	count_t	stime;		// accumulate system  time in clock ticks for per numa
 	count_t	utime;		// accumulate user    time in clock ticks for per numa
@@ -148,7 +156,7 @@ struct freqcnt {
         count_t maxfreq;/* frequency in MHz                    */
         count_t cnt;    /* number of clock ticks times state   */
         count_t ticks;  /* number of total clock ticks         */
-                        /* if zero, cnt is actul freq          */
+                        /* if zero, cnt is actual freq         */
 };
 
 struct percpu {
@@ -185,16 +193,17 @@ struct	cpustat {
 /************************************************************************/
 
 struct	perdsk {
-        char	name[MAXDKNAM];	/* empty string for last        */
-        count_t	nread;	/* number of read  transfers            */
-        count_t	nrsect;	/* number of sectors read               */
-        count_t	nwrite;	/* number of write transfers            */
-        count_t	nwsect;	/* number of sectors written            */
-        count_t	io_ms;	/* number of millisecs spent for I/O    */
-        count_t	avque;	/* average queue length                 */
-        count_t	ndisc;	/* number of discards (-1 = unavailable)*/
-        count_t	ndsect;	/* number of sectors discarded          */
-	count_t	cfuture[2];	/* reserved for future use	*/
+        char	name[MAXDKNAM];	/* empty string for last		*/
+        count_t	nread;		/* number of read  transfers		*/
+        count_t	nrsect;		/* number of sectors read		*/
+        count_t	nwrite;		/* number of write transfers		*/
+        count_t	nwsect;		/* number of sectors written		*/
+        count_t	io_ms;		/* number of millisecs spent for I/O	*/
+        count_t	avque;		/* average queue length			*/
+        count_t	ndisc;		/* number of discards (-1 = unavailable)*/
+        count_t	ndsect;		/* number of sectors discarded		*/
+        count_t	inflight;	/* number of inflight I/O		*/
+        count_t	cfuture[3];	/* reserved for future use		*/
 };
 
 struct dskstat {
@@ -400,6 +409,20 @@ struct ifbstat {
 	int		nrports;	// total number of IB ports
 	struct perifb   ifb[MAXIBPORT];
 };
+
+/************************************************************************/
+struct perllc {
+	unsigned char	id;
+	float		occupancy;
+	count_t		mbm_local;
+	count_t		mbm_total;
+};
+
+struct llcstat {
+	unsigned char	nrllcs;	        // total number of LLC
+	struct perllc   perllc[MAXLLC];
+};
+
 /************************************************************************/
 
 struct	sstat {
@@ -415,6 +438,7 @@ struct	sstat {
 	struct pressure	psi;
 	struct gpustat 	gpu;
 	struct ifbstat 	ifb;
+	struct llcstat  llc;
 
 	struct wwwstat	www;
 };
