@@ -152,6 +152,9 @@ rawwrite(time_t curtime, int numsecs,
 	if (supportflags & NETATOPD)
 		rr.flags |= RRNETATOPD;
 
+	if (supportflags & CGROUPV2)
+		rr.flags |= RRCGRSTAT;
+
 	if (supportflags & DOCKSTAT)
 		rr.flags |= RRDOCKSTAT;
 
@@ -277,6 +280,7 @@ rawwopen()
 	rh.ossub	= ossub;
 	rh.hertz	= hertz;
 	rh.pagesize	= pagesize;
+	rh.pidwidth	= getpidwidth();
 
 	memcpy(&rh.utsname, &utsname, sizeof rh.utsname);
 
@@ -531,6 +535,11 @@ rawread(void)
 	if (rh.pagesize)
 		pagesize = rh.pagesize;
 
+	if (rh.pidwidth)
+		pidwidth = rh.pidwidth;
+	else	
+		pidwidth = 5;
+
 	/*
 	** allocate a list for backtracking of rawrecord-offsets
 	*/
@@ -751,6 +760,11 @@ rawread(void)
 			else
 				supportflags &= ~NETATOPD;
 
+			if (rr.flags & RRCGRSTAT)
+				supportflags |=  CGROUPV2;
+			else
+				supportflags &= ~CGROUPV2;
+
 			if (rr.flags & RRDOCKSTAT)
 				supportflags |=  DOCKSTAT;
 			else
