@@ -83,6 +83,7 @@ static int	maxifblines = 999;  /* maximum infinibnd lines          */
 static int	maxnfslines = 999;  /* maximum nfs mount lines          */
 static int	maxcontlines = 999; /* maximum container lines          */
 static int	maxnumalines = 999; /* maximum numa      lines          */
+static int	maxllclines = 999;  /* maximum llc       lines          */
 
 static short	colorinfo   = COLOR_GREEN;
 static short	coloralmost = COLOR_CYAN;
@@ -264,6 +265,10 @@ generic_samp(time_t curtime, int nsecs,
 		if (sstat->cpunuma.nrnuma > 1 && maxnumalines > 0)
 			qsort(sstat->cpunuma.numa, sstat->cpunuma.nrnuma,
 			       sizeof sstat->cpunuma.numa[0], cpunumacompar);
+
+		if (sstat->llc.nrllcs > 1 && maxllclines > 0)
+			qsort(sstat->llc.perllc, sstat->llc.nrllcs,
+				sizeof sstat->llc.perllc[0], llccompar);
 	}
 
 	/*
@@ -274,6 +279,7 @@ generic_samp(time_t curtime, int nsecs,
 	while (1)
 	{
 		curline = 1;
+		genline[0] = '\0';
 
 	        /*
        	 	** prepare screen or file output for new sample
@@ -357,7 +363,7 @@ generic_samp(time_t curtime, int nsecs,
 		                  maxcpulines, maxgpulines, maxdsklines,
 				  maxmddlines, maxlvmlines,
 		                  maxintlines, maxifblines, maxnfslines,
-		                  maxcontlines, maxnumalines);
+		                  maxcontlines, maxnumalines, maxllclines);
 
 		/*
  		** if system-wide statistics do not fit,
@@ -380,7 +386,8 @@ generic_samp(time_t curtime, int nsecs,
 					maxdsklines, maxmddlines,
 		                        maxlvmlines, maxintlines,
 					maxifblines, maxnfslines,
-		                        maxcontlines, maxnumalines);
+		                        maxcontlines, maxnumalines,
+					maxllclines);
 
 			/*
  			** if system-wide statistics still do not fit,
@@ -461,13 +468,6 @@ generic_samp(time_t curtime, int nsecs,
 						attron(COLOR_PAIR(COLORINFO));
 
 					attron(A_BLINK);
-
-					printg("%*s",
-						(COLS-strlen(initmsg)-strlen(viewmsg)-5)/2, " ");
-				}
-				else
-				{
-					printg("        ");
 				}
 
        				printg(initmsg);
@@ -477,9 +477,17 @@ generic_samp(time_t curtime, int nsecs,
 					if (usecolors)
 						attroff(COLOR_PAIR(COLORINFO));
 					attroff(A_BLINK);
-				}
 
-				printg("     ");
+					printg("%*s", COLS - strlen(initmsg)
+					                   - strlen(viewmsg),
+					                     " ");
+				}
+				else
+				{
+					printg("%*s", 80 - strlen(initmsg)
+					                 - strlen(viewmsg),
+					                   " ");
+				}
 
 				if (screen)
 				{
@@ -1168,6 +1176,21 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** cgroup v2 info per process
+			   */
+			   case MPROCCGR:
+				if ( !(supportflags & CGROUPV2) )
+				{
+					statmsg = "No cgroup v2 figures "
+					          "available; request ignored!";
+					break;
+				}
+
+				showtype  = MPROCCGR;
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** own defined output per process
 			   */
@@ -2066,6 +2089,11 @@ generic_samp(time_t curtime, int nsecs,
 				            "statistics (now %d): ",
 					    maxnumalines, statline);
 
+				maxllclines =
+				  getnumval("Maximum lines for LLC "
+				            "statistics (now %d): ",
+					    maxllclines, statline);
+
 				if (interval && !paused && !rawreadflag)
 					alarm(3);  /* set short timer */
 
@@ -2549,6 +2577,9 @@ limitedlines(void)
 
 	if (maxnumalines == 999)	// default?
 		maxnumalines = 0;
+
+	if (maxllclines  == 999)	// default?
+		maxllclines = 0;
 }
 
 /*
@@ -2692,7 +2723,7 @@ generic_init(void)
 			if ( !(supportflags & NETATOP) )
 			{
 				fprintf(stderr, "Kernel module 'netatop' not "
-					          "active; request ignored!");
+					          "active; request ignored!\n");
 				sleep(3);
 				break;
 			}
@@ -2709,6 +2740,18 @@ generic_init(void)
 			showtype  = MPROCARG;
 			break;
 
+		   case MPROCCGR:
+			if ( !(supportflags & CGROUPV2) )
+			{
+				fprintf(stderr, "No cgroup v2 details "
+				          "available; request ignored!\n");
+				sleep(3);
+				break;
+			}
+
+			showtype  = MPROCCGR;
+			break;
+
 		   case MPROCOWN:
 			showtype  = MPROCOWN;
 			break;
@@ -2806,7 +2849,7 @@ generic_init(void)
        	** check if STDOUT is related to a tty or
        	** something else (file, pipe)
        	*/
-       	if ( isatty(1) )
+       	if ( isatty(fileno(stdout)) )
                	screen = 1;
        	else
              	screen = 0;
@@ -2822,10 +2865,9 @@ generic_init(void)
 		** to pipe or file), close it and duplicate stdout (tty)
 		** to stdin
 		*/
-       		if ( !isatty(0) )
+       		if ( !isatty(fileno(stdin)) )
 		{
-			(void) close(0);
-			(void) dup(1);
+			(void) dup2(fileno(stdout), fileno(stdin));
 		}
 
 		/*
@@ -2882,11 +2924,13 @@ static struct helptext {
 	{"\t'%c'  - memory details\n",				MPROCMEM},
 	{"\t'%c'  - disk details\n",				MPROCDSK},
 	{"\t'%c'  - network details\n",				MPROCNET},
-	{"\t'%c'  - GPU details\n",				MPROCGPU},
+	{"\t'%c'  - cgroups v2 details\n",			MPROCCGR},
 	{"\t'%c'  - scheduling and thread-group info\n",	MPROCSCH},
+	{"\t'%c'  - GPU details\n",				MPROCGPU},
 	{"\t'%c'  - various info (ppid, user/group, date/time, status, "
 	 "exitcode)\n",	MPROCVAR},
 	{"\t'%c'  - full command line per process\n",		MPROCARG},
+	{"\t'%c'  - cgroup v2 info per process\n",		MPROCCGR},
 	{"\t'%c'  - use own output line definition\n",		MPROCOWN},
 	{"\n",							' '},
 	{"Sort list of processes in order of:\n",		' '},
@@ -3089,6 +3133,8 @@ generic_usage(void)
 	                 "date/time)\n", MPROCVAR);
 	printf("\t  -%c  show command line per process\n",
 			MPROCARG);
+	printf("\t  -%c  show cgroup v2 info per process\n",
+			MPROCCGR);
 	printf("\t  -%c  show own defined process-info\n",
 			MPROCOWN);
 	printf("\t  -%c  show cumulated process-info per user\n",
@@ -3259,6 +3305,12 @@ do_maxnuma(char *name, char *val)
 	maxnumalines = get_posval(name, val);
 }
 
+void
+do_maxllc(char *name, char *val)
+{
+	maxllclines = get_posval(name, val);
+}
+
 struct colmap {
 	char 	*colname;
 	short	colval;
@@ -3392,6 +3444,10 @@ do_flags(char *name, char *val)
 			showtype  = MPROCARG;
 			break;
 
+		   case MPROCCGR:
+			showtype  = MPROCCGR;
+			break;
+
 		   case MPROCOWN:
 			showtype  = MPROCOWN;
 			break;
