@@ -349,11 +349,11 @@ sys_printdef *cpisyspdefs[] = {
         0
 };
 sys_printdef *cplsyspdefs[] = {
+	&syspdef_CPLNUMCPU,
 	&syspdef_CPLAVG1,
 	&syspdef_CPLAVG5,
 	&syspdef_CPLAVG15,
 	&syspdef_CPLCSW,
-	&syspdef_CPLNUMCPU,
 	&syspdef_CPLINTR,
 	&syspdef_BLANKBOX,
         0
@@ -371,15 +371,29 @@ sys_printdef *gpusyspdefs[] = {
 	&syspdef_BLANKBOX,
         0
 };
-sys_printdef *memsyspdefs[] = {
+sys_printdef *memsyspdefs1[] = {
 	&syspdef_MEMTOT,
 	&syspdef_MEMFREE,
+	&syspdef_BLANKBOX,
 	&syspdef_MEMCACHE,
 	&syspdef_MEMDIRTY,
 	&syspdef_MEMBUFFER,
+	&syspdef_BLANKBOX,
 	&syspdef_MEMSLAB,
 	&syspdef_RECSLAB,
 	&syspdef_BLANKBOX,
+	&syspdef_PAGETABS,
+	&syspdef_BLANKBOX,
+	&syspdef_HUPTOT,
+	&syspdef_HUPUSE,
+        0
+};
+sys_printdef *memsyspdefs2[] = {
+	&syspdef_NUMNUMA,
+	&syspdef_BLANKBOX,
+	&syspdef_TCPSOCK,
+	&syspdef_UDPSOCK,
+	&syspdef_BLANKBOX,
 	&syspdef_SHMEM,
 	&syspdef_SHMRSS,
 	&syspdef_SHMSWP,
@@ -387,10 +401,6 @@ sys_printdef *memsyspdefs[] = {
 	&syspdef_VMWBAL,
 	&syspdef_BLANKBOX,
 	&syspdef_ZFSARC,
-	&syspdef_BLANKBOX,
-	&syspdef_HUPTOT,
-	&syspdef_HUPUSE,
-	&syspdef_NUMNUMA,
         0
 };
 sys_printdef *swpsyspdefs[] = {
@@ -418,6 +428,8 @@ sys_printdef *pagsyspdefs[] = {
 	&syspdef_PAGSWIN,
 	&syspdef_PAGSWOUT,
 	&syspdef_OOMKILLS,
+	&syspdef_PAGPGIN,
+	&syspdef_PAGPGOUT,
 	&syspdef_BLANKBOX,
         0
 };
@@ -449,6 +461,13 @@ sys_printdef *cpunumasyspdefs[] = {
 	&syspdef_NUMANUMCPU,
         0
 };
+sys_printdef *llcsyspdefs[] = {
+	&syspdef_LLCMBMTOTAL,
+	&syspdef_LLCMBMLOCAL,
+	&syspdef_NUMLLC,
+	&syspdef_BLANKBOX,
+        0
+};
 sys_printdef *psisyspdefs[] = {
 	&syspdef_PSICPUSTOT,
 	&syspdef_PSIMEMSTOT,
@@ -482,6 +501,7 @@ sys_printdef *dsksyspdefs[] = {
 	&syspdef_DSKKBPERRD,
 	&syspdef_DSKKBPERWR,
 	&syspdef_DSKKBPERDS,
+	&syspdef_DSKINFLIGHT,
 	&syspdef_DSKAVQUEUE,
 	&syspdef_DSKAVIO,
 	&syspdef_BLANKBOX,
@@ -594,6 +614,7 @@ proc_printdef *allprocpdefs[]=
 	&procprt_SYSCPU,
 	&procprt_USRCPU,
 	&procprt_RUNDELAY,
+	&procprt_BLKDELAY,
 	&procprt_WCHAN,
 	&procprt_VGROW,
 	&procprt_RGROW,
@@ -659,6 +680,14 @@ proc_printdef *allprocpdefs[]=
 	&procprt_GPUMEMAVG,
 	&procprt_GPUGPUBUSY,
 	&procprt_GPUMEMBUSY,
+	&procprt_CGROUP_PATH,
+	&procprt_CGRCPUWGT,
+	&procprt_CGRCPUMAX,
+	&procprt_CGRCPUMAXR,
+	&procprt_CGRMEMMAX,
+	&procprt_CGRMEMMAXR,
+	&procprt_CGRSWPMAX,
+	&procprt_CGRSWPMAXR,
 	&procprt_SORTITEM,
         0
 };
@@ -690,6 +719,7 @@ proc_printpair netprocs[MAXITEMS];
 proc_printpair gpuprocs[MAXITEMS];
 proc_printpair varprocs[MAXITEMS];
 proc_printpair cmdprocs[MAXITEMS];
+proc_printpair cgrprocs[MAXITEMS];
 proc_printpair ownprocs[MAXITEMS];
 proc_printpair totusers[MAXITEMS];
 proc_printpair totprocs[MAXITEMS];
@@ -706,10 +736,12 @@ sys_printpair allcpuline[MAXITEMS];
 sys_printpair indivcpuline[MAXITEMS];
 sys_printpair cplline[MAXITEMS];
 sys_printpair gpuline[MAXITEMS];
-sys_printpair memline[MAXITEMS];
+sys_printpair memline1[MAXITEMS];
+sys_printpair memline2[MAXITEMS];
 sys_printpair swpline[MAXITEMS];
 sys_printpair memnumaline[MAXITEMS];
 sys_printpair cpunumaline[MAXITEMS];
+sys_printpair llcline[MAXITEMS];
 sys_printpair pagline[MAXITEMS];
 sys_printpair psiline[MAXITEMS];
 sys_printpair contline[MAXITEMS];
@@ -857,22 +889,8 @@ make_sys_prints(sys_printpair *ar, int maxn, const char *pairs,
 static void 
 init_proc_prints(count_t numcpu)
 {
-	int 	i, numdigits = 5;
 	char	linebuf[64];
-	FILE	*fp;
-
-	/*
-	** determine maximum number of digits for PID/TID
-	*/
-	if ( (fp = fopen("/proc/sys/kernel/pid_max", "r")) != NULL)
-	{
-                if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
-                {
-                        numdigits = strlen(linebuf) - 1;
-                }
-
-                fclose(fp);
-        }
+	int	i;
 
 	/*
 	** fill number of digits for various PID/TID columns
@@ -880,15 +898,15 @@ init_proc_prints(count_t numcpu)
 	*/
 	for (i=0; idprocpdefs[i] != 0; i++)
 	{
-		idprocpdefs[i]->width = numdigits;
+		idprocpdefs[i]->width = pidwidth;
 
-		if ( strlen(idprocpdefs[i]->head) < numdigits)
+		if ( strlen(idprocpdefs[i]->head) < pidwidth)
 		{
-			char *p = malloc(numdigits+1);
+			char *p = malloc(pidwidth+1);
 
 			ptrverify(p, "Malloc failed for formatted header\n");
 
-			sprintf(p, "%*s", numdigits, idprocpdefs[i]->head);
+			sprintf(p, "%*s", pidwidth, idprocpdefs[i]->head);
 			idprocpdefs[i]->head = p;
 		}
 	}
@@ -1096,14 +1114,14 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                 if (cplline[0].f == 0)
                 {
                     make_sys_prints(cplline, MAXITEMS,
+	                "CPLNUMCPU:7"
+	                "BLANKBOX:0 "
 	                "CPLAVG1:4 "
 	                "CPLAVG5:3 "
 	                "CPLAVG15:2 "
 	                "BLANKBOX:0 "
 	                "CPLCSW:6 "
-	                "CPLINTR:5 "
-	                "BLANKBOX:0 "
-	                "CPLNUMCPU:1",
+	                "CPLINTR:5 ",
 			cplsyspdefs, "builtin cplline",
 			sstat, &extra);
                 }
@@ -1125,29 +1143,43 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 			NULL, NULL);
                 }
 
-                if (memline[0].f == 0)
+                if (memline1[0].f == 0)
                 {
-                    make_sys_prints(memline, MAXITEMS,
+                    make_sys_prints(memline1, MAXITEMS,
 	                "MEMTOT:8 "
 	                "MEMFREE:9 "
-	                "MEMCACHE:7 "
-	                "MEMDIRTY:5 "
+	                "BLANKBOX:0 "
+	                "MEMCACHE:8 "
+	                "MEMDIRTY:6 "
 	                "MEMBUFFER:7 "
+	                "BLANKBOX:0 "
 	                "MEMSLAB:7 "
-	                "RECSLAB:4 "
+	                "RECSLAB:3 "
 	                "BLANKBOX:0 "
-	                "SHMEM:4 "
-	                "SHMRSS:4 "
-	                "SHMSWP:3 "
+	                "PAGETABS:4 "
 	                "BLANKBOX:0 "
-	                "VMWBAL:5 "
+	                "HUPTOT:5 "
+	                "HUPUSE:2 ",
+			memsyspdefs1, "builtin memline1",
+			sstat, &extra);
+                }
+
+                if (memline2[0].f == 0)
+                {
+                    make_sys_prints(memline2, MAXITEMS,
+	                "NUMNUMA:6 "
+	                "BLANKBOX:1 "
+	                "SHMEM:3 "
+	                "SHMRSS:3 "
+	                "SHMSWP:2 "
 	                "BLANKBOX:0 "
-	                "ZFSARC:6 "
+	                "TCPSOCK:5 "
+	                "UDPSOCK:2 "
 	                "BLANKBOX:0 "
-	                "HUPTOT:6 "
-	                "HUPUSE:3 "
-	                "NUMNUMA:7 ",
-			memsyspdefs, "builtin memline",
+	                "VMWBAL:4 "
+	                "BLANKBOX:0 "
+	                "ZFSARC:5 ",
+			memsyspdefs2, "builtin memline2",
 			sstat, &extra);
                 }
 
@@ -1206,6 +1238,17 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 			NULL, NULL);
                 }
 
+                if (llcline[0].f == 0)
+                {
+                    make_sys_prints(llcline, MAXITEMS,
+	                "LLCMBMTOTAL:9 "
+	                "LLCMBMLOCAL:8 "
+	                "NUMLLC:7 "
+	                "BLANKBOX:0 ",
+			llcsyspdefs, "builtin llcline",
+			sstat, &extra);
+                }
+
                 if (pagline[0].f == 0)
                 {
                     make_sys_prints(pagline, MAXITEMS,
@@ -1215,12 +1258,11 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "PAGCOMPACT:5 "
 			"NUMAMIGRATE:5"
 			"PGMIGRATE:6"
-	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
+	                "PAGPGIN:7 "
+	                "PAGPGOUT:7 "
 	                "PAGSWIN:5 "
-	                "PAGSWOUT:7 "
-			"OOMKILLS:8 ",
+	                "PAGSWOUT:8 "
+			"OOMKILLS:9 ",
 			pagsyspdefs, "builtin pagline",
 			sstat, &extra);
                 }
@@ -1269,6 +1311,7 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "DSKKBPERDS:4 "
                         "DSKMBPERSECRD:6 "
                         "DSKMBPERSECWR:6 "
+	                "DSKINFLIGHT:2 "
 	                "DSKAVQUEUE:1 "
 	                "DSKAVIO:6",
 			dsksyspdefs, "builtin dskline",
@@ -1451,7 +1494,7 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                 make_proc_prints(schedprocs, MAXITEMS, 
                         "PID:10 TID:6 CID:4 VPID:3 CTID:3 TRUN:7 TSLPI:7 "
 			"TSLPU:7 POLI:8 NICE:9 PRI:5 RTPR:9 CPUNR:8 ST:8 "
-			"EXC:8 S:8 RDELAY:8 WCHAN:5 SORTITEM:10 CMD:10",
+			"EXC:8 S:8 RDELAY:8 BDELAY:7 WCHAN:5 SORTITEM:10 CMD:10",
                         "built-in schedprocs");
 
                 make_proc_prints(dskprocs, MAXITEMS, 
@@ -1485,6 +1528,11 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                         "PID:10 TID:4 S:8 SORTITEM:10 COMMAND-LINE:10", 
                         "built-in cmdprocs");
 
+                make_proc_prints(cgrprocs, MAXITEMS, 
+                        "PID:10 CPUWGT:9 CPUMAX:9 CPUMAXR:9 MEMMAX:8 MMMAXR:8 "
+			"SWPMAX:7 SWMAXR:7 SORTITEM:6 CMD:9 CGROUP-PATH:10", 
+                        "built-in cgrprocs");
+
                 make_proc_prints(totusers, MAXITEMS, 
                         "NPROCS:10 SYSCPU:9 USRCPU:9 VSIZE:6 "
                         "RSIZE:8 PSIZE:8 LOCKSZ:3 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
@@ -1554,6 +1602,10 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                 showhdrline(cmdprocs, curlist, totlist, *showorder, autosort);
                 break;
 
+           case MPROCCGR:
+                showhdrline(cgrprocs, curlist, totlist, *showorder, autosort);
+                break;
+
            case MPROCOWN:
                 showhdrline(ownprocs, curlist, totlist, *showorder, autosort);
                 break;
@@ -1586,6 +1638,7 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 #define	FORMCID	"CID:5 "
 #define	FORMCPU	"SYSCPU:9 USRCPU:9 "
 #define	FORMDEL	"RDELAY:4 "
+#define	FORMBDL	"BDELAY:4 "
 #define FORMMEM	"VGROW:8 RGROW:8 "
 #define FORMDSK	"RDDSK:7 CWRDSK:7 "
 #define FORMNET	"RNET:6 SNET:6 "
@@ -1618,6 +1671,9 @@ make_proc_dynamicgen()
 	memcpy(p, FORMDEL, sizeof FORMDEL -1);
 	p += sizeof FORMDEL -1;
 
+	memcpy(p, FORMBDL, sizeof FORMBDL -1);
+	p += sizeof FORMBDL -1;
+
 	memcpy(p, FORMMEM, sizeof FORMMEM -1);
 	p += sizeof FORMMEM -1;
 
@@ -1796,6 +1852,10 @@ priproc(struct tstat **proclist, int firstproc, int lastproc, int curline,
                         showprocline(cmdprocs, curstat, perc, nsecs, avgval);
                         break;
 
+                   case MPROCCGR:
+                        showprocline(cgrprocs, curstat, perc, nsecs, avgval);
+                        break;
+
                    case MPROCOWN:
                         showprocline(ownprocs, curstat, perc, nsecs, avgval);
                         break;
@@ -1835,7 +1895,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         int fixedhead, struct sselection *selp, char *highorderp,
         int maxcpulines, int maxgpulines, int maxdsklines, int maxmddlines,
 	int maxlvmlines, int maxintlines, int maxifblines,
-	int maxnfslines, int maxcontlines, int maxnumalines)
+	int maxnfslines, int maxcontlines, int maxnumalines, int maxllclines)
 {
         extraparam      extra;
         int             lin;
@@ -2024,7 +2084,10 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 	if (screen)
 	        move(curline, 0);
 
-        showsysline(memline, sstat, &extra, "MEM", badness);
+        showsysline(memline1, sstat, &extra, "MEM", badness);
+        curline++;
+
+        showsysline(memline2, sstat, &extra, "MEM", badness);
         curline++;
 
         /*
@@ -2147,6 +2210,27 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 		}
 	}
 
+	/*
+ 	** LLC statistics (if supported by kernel and
+	** pseudo filesystem mounted)
+	*/
+	for (extra.index=0, lin=0;
+			extra.index < sstat->llc.nrllcs && lin < maxllclines;
+			extra.index++)
+	{
+		if (fixedhead     	        	  	||
+		    sstat->llc.perllc[extra.index].mbm_local	||
+		    sstat->llc.perllc[extra.index].mbm_total	  )
+		{
+			if (screen)
+				move(curline, 0);
+
+			showsysline(llcline, sstat, &extra, "LLC", 0);
+			curline++;
+			lin++;
+		}
+	}
+
         /*
         ** PAGING statistics
         */
@@ -2155,6 +2239,10 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
             sstat->mem.pgsteal    	||
             sstat->mem.allocstall 	||
             sstat->mem.compactstall 	||
+            sstat->mem.pgins      	||
+            sstat->mem.pgouts     	||
+            sstat->mem.tcpsock     	||
+            sstat->mem.udpsock     	||
             sstat->mem.swins      	||
             sstat->mem.swouts     	||
             sstat->mem.oomkills > 0   	||
@@ -2927,6 +3015,24 @@ cpunumacompar(const void *a, const void *b)
 	return  0;
 }
 
+int
+llccompar(const void *a, const void *b)
+{
+        const struct perllc *ca = a;
+        const struct perllc *cb = b;
+
+        register count_t aused = ca->mbm_local + ca->mbm_total;
+        register count_t bused = cb->mbm_local + cb->mbm_total;
+
+        if (aused < bused)
+                return  1;
+
+        if (aused > bused)
+                return -1;
+
+        return  0;
+}
+
 /*
 ** handle modifications from the /etc/atoprc and ~/.atoprc file
 */
@@ -3054,7 +3160,7 @@ do_owngpuline(char *name, char *val)
 void
 do_ownmemline(char *name, char *val)
 {
-        make_sys_prints(memline, MAXITEMS, val, memsyspdefs, name,
+        make_sys_prints(memline1, MAXITEMS, val, memsyspdefs1, name,
 					NULL, NULL);
 }
 
@@ -3086,6 +3192,13 @@ do_owncpunumaline(char *name, char *val)
 					NULL, NULL);
 }
 
+void
+do_ownllcline(char *name, char *val)
+{
+        make_sys_prints(llcline, MAXITEMS, val, llcsyspdefs, name,
+					NULL, NULL);
+}
+
 void
 do_owndskline(char *name, char *val)
 {
