@@ -132,7 +132,8 @@ extern sys_printdef *prcsyspdefs[];
 extern sys_printdef *cpusyspdefs[];
 extern sys_printdef *cpisyspdefs[];
 extern sys_printdef *cplsyspdefs[];
-extern sys_printdef *memsyspdefs[];
+extern sys_printdef *memsyspdefs1[];
+extern sys_printdef *memsyspdefs2[];
 extern sys_printdef *swpsyspdefs[];
 extern sys_printdef *pagsyspdefs[];
 extern sys_printdef *numasyspdefs[];
@@ -202,6 +203,7 @@ extern sys_printdef syspdef_SHMRSS;
 extern sys_printdef syspdef_SHMSWP;
 extern sys_printdef syspdef_VMWBAL;
 extern sys_printdef syspdef_ZFSARC;
+extern sys_printdef syspdef_PAGETABS;
 extern sys_printdef syspdef_HUPTOT;
 extern sys_printdef syspdef_HUPUSE;
 extern sys_printdef syspdef_SWPTOT;
@@ -236,12 +238,19 @@ extern sys_printdef syspdef_NUMACPUIDLE;
 extern sys_printdef syspdef_NUMACPUWAIT;
 extern sys_printdef syspdef_NUMACPUSTEAL;
 extern sys_printdef syspdef_NUMACPUGUEST;
+extern sys_printdef syspdef_LLCMBMTOTAL;
+extern sys_printdef syspdef_LLCMBMLOCAL;
+extern sys_printdef syspdef_NUMLLC;
 extern sys_printdef syspdef_PAGSCAN;
 extern sys_printdef syspdef_PAGSTEAL;
 extern sys_printdef syspdef_PAGSTALL;
 extern sys_printdef syspdef_PAGCOMPACT;
 extern sys_printdef syspdef_NUMAMIGRATE;
 extern sys_printdef syspdef_PGMIGRATE;
+extern sys_printdef syspdef_PAGPGIN;
+extern sys_printdef syspdef_PAGPGOUT;
+extern sys_printdef syspdef_TCPSOCK;
+extern sys_printdef syspdef_UDPSOCK;
 extern sys_printdef syspdef_PAGSWIN;
 extern sys_printdef syspdef_PAGSWOUT;
 extern sys_printdef syspdef_OOMKILLS;
@@ -269,6 +278,7 @@ extern sys_printdef syspdef_DSKMBPERSECWR;
 extern sys_printdef syspdef_DSKKBPERRD;
 extern sys_printdef syspdef_DSKKBPERWR;
 extern sys_printdef syspdef_DSKKBPERDS;
+extern sys_printdef syspdef_DSKINFLIGHT;
 extern sys_printdef syspdef_DSKAVQUEUE;
 extern sys_printdef syspdef_DSKAVIO;
 extern sys_printdef syspdef_NETTRANSPORT;
@@ -420,8 +430,16 @@ extern proc_printdef procprt_GPUGPUBUSY;
 extern proc_printdef procprt_GPUMEMBUSY;
 extern proc_printdef procprt_SORTITEM;
 extern proc_printdef procprt_RUNDELAY;
+extern proc_printdef procprt_BLKDELAY;
 extern proc_printdef procprt_WCHAN;
-
+extern proc_printdef procprt_CGROUP_PATH;
+extern proc_printdef procprt_CGRCPUWGT;
+extern proc_printdef procprt_CGRCPUMAX;
+extern proc_printdef procprt_CGRCPUMAXR;
+extern proc_printdef procprt_CGRMEMMAX;
+extern proc_printdef procprt_CGRMEMMAXR;
+extern proc_printdef procprt_CGRSWPMAX;
+extern proc_printdef procprt_CGRSWPMAXR;
 
 
 //extern char *procprt_NRDDSK_ae(struct tstat *, int, int);
